<?php
namespace Controllers;

use \Zeedhi\Framework\Report\ReportService;
use \Zeedhi\Framework\DTO;
use \Zeedhi\Framework\DTO\Response\Error;
use \Zeedhi\Framework\DB\StoredProcedure\Param;
use \Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;

class SimulacaoGeracaoPlanoVoo extends \Zeedhi\Framework\Controller\Report {

    protected $environment;
    protected $connection;
    protected $MessagesProcedure;
    protected $birtStrategy;
    
    public function __construct(ReportService $reportService, $environment, $connection, $MessagesProcedure, $birtStrategy) {
        $this->environment = $environment;
        $this->connection = $connection;
        $this->MessagesProcedure = $MessagesProcedure;
        $this->birtStrategy = $birtStrategy;
        parent::__construct($reportService);
    }
    
    protected $reportMapping = array (
        'ATE00003' =>
        array (
          'strategy' => 'BIRT',
          'parameterMapping' =>
          array (
            "CDFILIAL"           => "CDFILIAL",
            "NRERROSIMULAPLAVOO" => "NRERROSIMULAPLAVOO",
            "NRORG"              => "NRORG"
          ),
          'reportName'
        )
    );
  
    public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        $row['NRORG']  = $this->environment->getNrOrgTrab();
        $row['__report_name'] = $row['REPORTNAME'];
        $this->birtStrategy->setReportFormat($row['REPORTFORMAT']);
        $nrerrosimulaplavoo = $this->SIMULA_GERAPLAVOO($row, $response);
        $this->atualizaDescricaoErro($nrerrosimulaplavoo, $row['CDFILIAL'], $response);
        $row['NRERROSIMULAPLAVOO'] = $nrerrosimulaplavoo;
    }
  
    private function SIMULA_GERAPLAVOO($row, $response){
        $procedure = new StoredProcedure($this->connection, 'SIMULA_GERAPLAVOO');                                         //SIMULA_GERAPLAVOO
        $procedure->addParam(new Param('P_CDFILIAL',           Param::PARAM_INPUT, $row["CDFILIAL"]));                    //   (P_CDFILIAL IN VARCHAR2,
        $procedure->addParam(new Param('P_CDCIA',              Param::PARAM_INPUT, $row["CDCIA"]));                       //   P_CDCIA IN VARCHAR,
        $procedure->addParam(new Param('P_NRVOO',              Param::PARAM_INPUT, $row["NRVOO"]));                       //   P_NRVOO IN VARCHAR2, => favor criar a chave de seleção como _CDCIA + | + NRVOO + | CDTRECHO_
        $procedure->addParam(new Param('P_DATAINI',            Param::PARAM_INPUT, $row["DTINICIAL"]));                   //   P_DATAINI IN DATE,
        $procedure->addParam(new Param('P_DATAFIN',            Param::PARAM_INPUT, $row["DTFINAL"]));                     //   P_DATAFIN IN DATE, 
        $procedure->addParam(new Param('P_CDOPERADOR',         Param::PARAM_INPUT, $this->environment->getCdOperador())); //   P_CDOPERADOR IN VARCHAR2,
        $procedure->addParam(new Param('P_NRORG',              Param::PARAM_INPUT, $this->environment->getNrOrgTrab()));  //   P_NRORG IN NUMBER,
        $procedure->addParam(new Param('P_NRERROSIMULAPLAVOO', Param::PARAM_OUTPUT, null, null, 20000));                      //   P_NRERROSIMULAPLAVOO OUT NUMBER)
        $result = $procedure->execute();
        if($result['P_NRERROSIMULAPLAVOO'] == null){
            $response->setError(new Error("Não há dados a serem listados", 500));
        } else{
            return $result['P_NRERROSIMULAPLAVOO'];
        }
    }
  
    private function getDataSetErros($nrerrosimulaplavoo, $cdfilial, $response){
        
        try{
            $sql = "SELECT CDFILIAL,
                           NRERROSIMULAPLAVOO,
                           NRSEQUENCIAL,
                           CDCIA,
                           NRVOO,
                           CDTRECHO,
                           DTHRPLAVOOPV,
                           DSERROBANCO,
                           DSERROTRADUZIDO,
                           NRORG,
                           IDATIVO,
                           DTINCLUSAO,
                           CDOPERULTATU,
                           DTULTATU,
                           CDOPERINCLUSAO
                      FROM ERROSIMULAPLAVOO 
                     WHERE CDFILIAL = :CDFILIAL
                       AND NRERROSIMULAPLAVOO = :NRERROSIMULAPLAVOO";
                    
            $params = array(
                'CDFILIAL'            => $cdfilial,
                'NRERROSIMULAPLAVOO'  => $nrerrosimulaplavoo
            );
            
            $query = $this->connection->prepare($sql);
            
            $query->execute($params);
            
            $dataSet = $query->fetchAll(\PDO::FETCH_ASSOC);
            
            return $dataSet;
        } catch (\Exception $e){
    		$response->setError(new Error($e->getMessage(),500));
    	}
    	
    }
  
    private function atualizaDescricaoErro($nrerrosimulaplavoo, $cdfilial, $response){
        
        $dataSet = $this->getDataSetErros($nrerrosimulaplavoo, $cdfilial, $response);
        try{
            $this->connection->beginTransaction();
            foreach($dataSet as $row){
                $sql = "UPDATE ERROSIMULAPLAVOO 
                           SET DSERROTRADUZIDO = :DSERROTRADUZIDO,
                               DTINCLUSAO      = TRUNC(SYSDATE),
                               CDOPERULTATU    = :OPERADOR,
                               DTULTATU        = TRUNC(SYSDATE),
                               CDOPERINCLUSAO  = :OPERADOR
                         WHERE CDFILIAL           = :CDFILIAL
                           AND NRERROSIMULAPLAVOO = :NRERROSIMULAPLAVOO
                           AND NRSEQUENCIAL       = :NRSEQUENCIAL
                           AND CDCIA              = :CDCIA
                           AND NRVOO              = :NRVOO
                           AND CDTRECHO           = :CDTRECHO
                           AND DTHRPLAVOOPV       = :DTHRPLAVOOPV
                           AND NRORG              = :NRORG";
                $params = array(
                    'DSERROTRADUZIDO' => $this->MessagesProcedure->convertMessage($row['DSERROBANCO']),
                    'CDFILIAL' => $row['CDFILIAL'],
                    'NRERROSIMULAPLAVOO' => $row['NRERROSIMULAPLAVOO'],
                    'NRSEQUENCIAL' => $row['NRSEQUENCIAL'],
                    'CDCIA' => $row['CDCIA'],
                    'NRVOO' => $row['NRVOO'],
                    'CDTRECHO' => $row['CDTRECHO'],
                    'DTHRPLAVOOPV' => $row['DTHRPLAVOOPV'],
                    'OPERADOR' => $this->environment->getCdOperador(),
                    'NRORG' => $this->environment->getNrOrg()
                );
                
                $query = $this->connection->prepare($sql);
                $query->execute($params);
            }
            $this->connection->commit();
        } catch (\Exception $e){
            $response->setError(new Error($this->MessagesProcedure->convertMessage($e->getMessage()), 500));
            $this->connection->rollback();
        }
      
    }
  
}