var MenuController = function($scope, templateManager, ZhLogger, ScreenService, $timeout, eventAggregator) {

	$scope.menuClick = function(menu) {
		var event;
		if (Util.isFunction(menu.click)) {
			event = menu.click();
		}
		if (!event || !event.isDefaultPrevented()) {
			if (menu.historyMenu) {
				buildMenusFromHistory(menu);
			}
			if (menu.menus) {
				if (Util.isDesktop()) {
					toggleMenu(menu);
				} else {
					if (!menu.menus.length) {
						menu.menus[0] = {};
					}
					menu.menus[0].context = {
						label: menu.label,
						menus: $scope.subMenu
					};

					var breadcrumb = [];
					var subMenu = $scope.subMenu;
					while (subMenu) {
						breadcrumb.unshift({
							label: subMenu[0].context.label,
							menus: subMenu
						});
						subMenu = subMenu[0].context.menus;
					}
					breadcrumb.unshift({
						label: ScreenService.i18n('Home'),
						menus: undefined
					});
					menu.menus[0].context.breadcrumb = breadcrumb;

					$scope.subMenu = menu.menus;
					$scope.labelMenu = $scope.subMenu[0].context.label;
					$scope.breadcrumb = $scope.subMenu[0].context.breadcrumb;
					$scope.showSubMenu = true;
				}
			} else {
				if (!menu.disabled) {
					$('.zh-input-search-menu').blur();
					if (menu.windowName) {
						ScreenService.toggleSideMenu();
						ScreenService.openWindow(menu.windowName);
					} else if (menu.link) {
						ScreenService.openLink(menu.link.url, menu.link.newWindow);
					} else if (!menu.events || menu.events.length === 0) {
						ZhLogger.warning('Menu item missing configuration!');
					}
					if (menu.historySubMenu) {
						$timeout($scope.menuBackClick, 500);
					}
				}
			}
		}
	};

	$scope.menuBackClick = function(item) {
		if (!item && !Util.isDesktop()) {
			$scope.subMenu = $scope.subMenu[0].context.menus ? $scope.subMenu[0].context.menus : void 0;
		} else {
			$scope.subMenu = (item && item.menus) ? item.menus : void 0;
		}
		$scope.labelMenu = $scope.subMenu ? $scope.subMenu[0].context.label : void 0;
		$scope.breadcrumb = $scope.subMenu ? $scope.subMenu[0].context.breadcrumb : [];
		$scope.showSubMenu = !!$scope.subMenu;
	};

	$scope.menuAboutClick = function(evt) {
		$('#tooltip-content').hide();
		ScreenService.showAbout();
	};

	$scope.menuReportIssueClick = function(evt){
		eventAggregator.publish('ReportBugOnClick', evt);
		if(!evt || !evt.isDefaultPrevented()) {
			eventAggregator.publish('OpenReportBug');
		}
	};

	function toggleMenu(menu) {
		if (menu.opened) {
			closeSubItems(menu);
		}
		menu.opened = !menu.opened;
	}

	function closeSubItems(menu) {
		if (menu.menus) {
			menu.menus.forEach(function(item) {
				item.opened = false;
				closeSubItems(item);
			});
		}
	}

	$scope.plainMenuData = undefined;
	$scope.templateManager = templateManager;

	var historyMenuItem = Util.extend(new Menu(), {
		historyMenu: true,
		label: ScreenService.i18n("Opened windows"),
		icon: "history"
	});

	function buildHistoryMenu() {
		if (templateManager.project.showHistoryMenu !== false) {

			if (!templateManager.project.historyMenuGroup) {
				templateManager.project.groupMenu.push(Util.extend(new Menu(), {
					name: "zhHistory",
					label: ScreenService.i18n("History"),
					hasChild: true,
					menus: []
				}));

				templateManager.project.historyMenuGroup = "zhHistory";
			}

			var historyGroup;
			templateManager.project.groupMenu.some(function(item) {
				if (item.name == templateManager.project.historyMenuGroup) {
					historyGroup = item;
					return true;
				}
			});

			// add historyItem to group chosen on containers.json
			if (historyGroup && historyGroup.menus) {
				historyGroup.menus.push(historyMenuItem);
				historyMenuItem.parent = historyGroup;
			}

		}
	}
	$scope.$on("openHistoryMenu", function() {
		if (templateManager.container.showMenu && templateManager.container.showHistoryMenu !== false) {
			// clean filter
			$scope.menuFilter = undefined;
			// reset variables => root menu
			$scope.subMenu = undefined;
			$scope.labelMenu = undefined;
			$scope.breadcrumb = [];

			if (!ScreenService.isSideMenuOpen()) {
				ScreenService.toggleSideMenu();
			}
			if (!historyMenuItem.opened) {
				$scope.menuClick(historyMenuItem);
			}
			$('.menu-item').removeClass('selected');
			$('.menu-item.history-menu').addClass('selected');
			templateManager.updateTemplate();
		}
	});
	templateManager.projectPromise.then(buildHistoryMenu);

	function buildMenusFromHistory(menu) {
		menu.menus = [];
		templateManager.historyItems.map(function(item) {
			menu.menus.unshift(Util.extend(new Menu(), {
				historySubMenu: true,
				label: templateManager.containers[item].label,
				windowName: item
			}));
		});
	}

	var buildMenuData = function() {
		templateManager.project.groupMenu
			.filter(function(item) {
				return item.aclIsVisible && item.isVisible;
			})
			.forEach(function(item) {
				buildMenuItems(item.menus);
			});
	};

	var buildMenuItems = function(menus, parentLabel) {
		menus.forEach(function(item) {
			if (item.menus && item.menus.length > 0) {
				buildMenuItems(item.menus, (parentLabel ? parentLabel + ' » ' : '') + item.label);
			} else if (!item.historySubMenu && item.aclIsVisible && item.isVisible) {
				$scope.plainMenuData.push({
					parentLabel: parentLabel,
					label: item.label,
					icon: item.icon,
					link: item.link,
					disabled: item.disabled,
					windowName: item.windowName,
					click: item.click
				});
			}
		});
	};

	$scope.$watch('menuFilter', function(val) {
		if (val && !$scope.plainMenuData) {
			$scope.plainMenuData = [];
			buildMenuData();
		}
	});

	this.menuSearch = function(item) {
		var filter = $scope.menuFilter;
		var itemLabel = ScreenService.i18n(item.label);
		if (filter && !Util.isUndefined(itemLabel)) {
			return itemLabel.indexOfLatin(filter.toLowerCase()) !== -1;
		} else {
			return !Util.isUndefined(itemLabel);
		}
	};

	this.menuOrder = function(menu){
		if(!templateManager.project.orderMenuOnSearch){
			return true;
		}
		return menu.label;
	};
};

ZeedhiController.controller('MenuController', MenuController);